package kernel.file;

import java.io.File;
import java.io.PipedInputStream;
import java.util.ArrayList;
import java.util.List;

import kernel.relation.Relation;
import kernel.service.CustomException;

/**
 * Description: File manager for work with files and folder.
 * 
 * @author david.kosek
 * @date 2. 12. 2014 15:11:38
 * @version $Rev$
 */
public class FileManager implements IFileManager {

	/**
	 * Declarate output
	 */
	private OutputUtility out;

	/**
	 * Declarate input
	 */
	private InputUtility in;

	/**
	 * Declarate relation for get current path
	 */
	private Relation relation;

	@Override
	public File getFile(final String aAbsolutePathToFile) {

		if (aAbsolutePathToFile.trim().length() == 0) {
			return null;
		}

		File file = new File(aAbsolutePathToFile);

		if (file.exists()) {
			return file;
		}
		return null;
	}

	@Override
	public String getCurrentPath() {
		return relation.getPath();
	}

	@Override
	public Boolean createFolder(final String aNameFolder) {

		if (aNameFolder.trim().length() == 0) {
			return false;
		}

		File newFolder = new File(aNameFolder);

		if (!newFolder.exists()) {
			try {
				newFolder.mkdir();
				return true;
			}
			catch (SecurityException se) {
				CustomException.printException(se, "Cannot create new folder.");
				return false;
			}
		}
		return false;
	}

	@Override
	public boolean deleteFile(String aName, boolean aRecursively) {
		if (aName.trim().length() == 0) {
			return false;
		}

		File file = new File(aName);

		if (file.exists()) {
			if (file.isDirectory() && aRecursively) {
				for (File c : file.listFiles())
					c.delete();
			}
			return file.delete();
		}
		return false;
	}

	@Override
	public List<File> loadAllFileFromDir(final String aPath) {
		return this.resolveCriteriaAndGetResult(aPath, "file");
	}

	@Override
	public List<File> loadAllFolderFromDir(final String aPath) {
		return this.resolveCriteriaAndGetResult(aPath, "folder");
	}

	@Override
	public List<File> loadAllFromDir(final String aPath) {
		return this.resolveCriteriaAndGetResult(aPath, "fileAndFolder");
	}

	@Override
	public List<File> loadAllAndHiddenFromDir(final String aPath) {
		return this.resolveCriteriaAndGetResult(aPath, "allAndHidden");
	}

	/**
	 * Loading request data from dir/folder to list
	 * 
	 * @param aPath request path
	 * @param aCondition condition for loading data
	 * @return result list of file
	 */
	private List<File> resolveCriteriaAndGetResult(final String aPath, final String aCondition) {
		String path = aPath;

		if (path.trim().length() == 0) {
			path = this.getCurrentPath();
		}

		File folder = new File(path);
		File[] listOfFiles = folder.listFiles();
		List<File> listOfFile = new ArrayList<File>();

		switch (aCondition) {
		case "file":
			for (File file : listOfFiles) {
				if (file.isFile() && !file.isHidden()) {
					listOfFile.add(file);
				}
			}
			break;
		case "folder":
			for (File file : listOfFiles) {
				if (file.isDirectory() && !file.isHidden()) {
					listOfFile.add(file);
				}
			}
			break;
		case "fileAndFolder":
			for (File file : listOfFiles) {
				if ((file.isFile() || file.isDirectory()) && !file.isHidden()) {
					listOfFile.add(file);
				}
			}
			break;
		case "allAndHidden":
			for (File file : listOfFiles) {
				if (file.isFile() || file.isDirectory() || file.isHidden()) {
					listOfFile.add(file);
				}
			}
			break;
		default:
			break;
		}

		return listOfFile;
	}

	@Override
	public Boolean openFileForWrite(String aNameFile, Boolean aAppend, PipedInputStream aPipeIn) {

		if (aNameFile.trim().length() == 0) {
			return false;
		}

		if (!aAppend) {
			out = new OutputUtility(aPipeIn, aNameFile, false);
			return true;
		}
		else if (aAppend) {
			out = new OutputUtility(aPipeIn, aNameFile, true);
			return true;
		}
		else {
			System.err.println("Unknow method for work with file.");
			return false;
		}
	}

	@Override
	public Boolean openFileForRead(String aNameFile) {
		if (aNameFile.trim().length() == 0) {
			return false;
		}
		in = new InputUtility(aNameFile);
		return true;
	}

	@Override
	public void writeToFile(String aText) {
		out.write(aText);
	}

	@Override
	public void closeWriteToFile() {
		out.close();
	}

	@Override
	public String readFromFile() {
		return in.read();
	}

	@Override
	public void closeReadToFile() {
		in.close();
	}

}
